;----------------------------------------------------------------------
;Adds Bearing and Distance to a line
;----------------------------------------------------------------------

(vl-load-com)

(defun rh:R2D (r) (* 180.0 (/ r pi)))

(defun gc:round (num prec) (if (zerop (setq prec (abs prec))) num (* prec (fix ((if (minusp num) - +) (/ num prec) 0.5)))))

(defun rh:midpoint ( pt1 pt2 / pt3 ) (setq pt3 (mapcar '(lambda (x y) (/ (+ x y) 2)) pt1 pt2)))

(defun rh:2azimuth ( a_brg / d a_azi)
  (setq d (fix a_brg))
  (cond ((and (>= d 0) (< d 90)) (setq a_azi (- 90 a_brg)))
        ((and (>= d 90) (< d 180)) (setq a_azi (- 360 (- a_brg 90))))
        ((and (>= d 180) (< d 270)) (setq a_azi (- 270 (- a_brg 180))))
        ((and (>= d 270) (< d 360)) (setq a_azi (- 90 (- a_brg 360))))
  )
  (setq d a_azi)
);_end_defun

; converts radian brg to dms text
(defun rh:2dms ( r_brg rnd / d_brg d m s b_str)
  (setq d_brg (rh:2azimuth (rh:R2D r_brg))
        d (fix d_brg)
        m (fix (* (rem d_brg 1.0) 60))
        s (gc:round (* (rem (* (rem d_brg 1.0) 60) 1.0) 60) rnd)
  );_end_setq
  (if (>= s 59.5) (setq m (1+ m) s 0.0))
  (if (= m 60) (setq d (1+ d) m 0))
  (if (>= d 360) (setq d (- d 360)))
  (setq s (rtos s 2 0) m (itoa m) d (itoa d))
  (if (< (atoi s) 10) (setq s (strcat "0" s)))
  (if (< (atoi m) 10) (setq m (strcat "0" m)))
  (while (< (strlen d) 3) (setq d (strcat "0" d)))
  (setq b_str (strcat d "%%d" m "'" s "\""))
);_end_defun

(defun rh:em_txt ( pt txt lyr ang d72 d73)
  (entmakex (list '(0 . "TEXT") '(100 . "AcDbEntity") '(100 . "AcDbText")
                 (cons 8 lyr) (cons 50 ang) (cons 7 (getvar 'textstyle)) (cons 1 txt)
                 (cons 10 pt) (cons 40 (getvar 'textsize)) (cons 72 d72) (cons 11 pt) (cons 73  d73)
            )
  )
);end_defun

;;BEGIN MAIN ROUTINE
(defun C:BD ( / *error* sv_lst sv_vals tht b_lyr_lst d_lyr_lst lyr_idx d_rnd a_rnd lans pik lyr_e l_lst lt dv txt b_lyr d_lyr l_obj l_ang a_txt l_txt m_pt i_ang d_pt)

  (defun *error* ( msg )
    (mapcar 'setvar sv_lst sv_vals)
    (if (not (wcmatch (strcase msg) "*BREAK*,*CANCEL*,*EXIT*")) (princ (strcat "\nOops an Error : " msg " occurred.")))
    (princ)
  );_end_*error*_defun

  ;(setq sv_lst (list 'cmdecho 'osmode 'dynmode 'dynprompt)
  (setq sv_lst (list 'cmdecho 'osmode 'dynmode)
        sv_vals (mapcar 'getvar sv_lst)
        tht (getvar 'textsize)
  );end_setq

  (mapcar 'setvar sv_lst '(0 0 3 1))

;; User Variables
  (setq b_lyr_lst (list "A-BRNG" "A-BRNG-COPD") ;; Bearing Layer list (first item is always default)
        d_lyr_lst (list "A-DIST" "A-DIST-COPD") ;; Distance Layer list (first item is always default)
        lyr_idx 0                               ;; Index for the above lists PLEASE DON'T CHANGE
        d_rnd 2                                 ;; Rounding for distance (Integer, number of decimal places)
        a_rnd 10.0                              ;; Rounding for angles (Real Seconds of Arc)
  );end_setq

  (initget "Default Copd")
  (setq lans (cond ( (getkword (strcat "\nUse Default (" (nth lyr_idx b_lyr_lst)" & " (nth lyr_idx d_lyr_lst) ") or Copied Layers : ? [Default/Copd] <Default>"))) ("Default")))
  (if (= lans "Copd") (setq lyr_idx 1 pik "Default") (setq pik "Copd"))

  (setq b_lyr (nth lyr_idx b_lyr_lst) d_lyr (nth lyr_idx d_lyr_lst))

  (foreach lyr (list b_lyr d_lyr) (if (not (tblsearch "layer" lyr)) (setq lyr_e T l_lst (cons lyr l_lst))))

  (cond (lyr_e
          (if (= (length l_lst) 2)  (setq lt "layers" dv " , ") (setq lt "layer" dv ""))
          (setq txt (strcat "MISSING LAYERS\n\nOption " lans " " lt " : "))
          (mapcar '(lambda (x) (setq txt (strcat txt x dv))) l_lst)
          (setq txt (vl-string-right-trim " ," txt))
          (alert (strcat txt "\n\nPlease rectify missing " lt " or re-run\nand select " pik " option"))
        )
  );end_cond (missing layers)

  (while (and (not lyr_e) (setq sel (entsel "\rSelect Line Entity to Label : ")))
    (setq l_obj (vlax-ename->vla-object (car sel)))
    (setq l_ang (vlax-get-property l_obj 'angle)
          a_txt (rh:2dms l_ang a_rnd)
          l_txt (strcat (rtos (vlax-get-property l_obj 'length) 2 d_rnd))
          m_pt (rh:midpoint (vlax-get l_obj 'startpoint) (vlax-get l_obj 'endpoint))
    );_end_setq
    (setq i_ang l_ang)
    (if (and (>= l_ang (* pi 0.5)) (< l_ang (* pi 1.5)))
      (setq i_ang (- l_ang pi) d_pt (polar m_pt (- i_ang (* pi 0.5)) (* tht 0.3)))
      (setq d_pt (polar m_pt (- i_ang (* pi 0.5)) (* tht 0.3)))
    );end_if
    (rh:em_txt m_pt a_txt b_lyr i_ang 1 1)
    (rh:em_txt d_pt l_txt d_lyr i_ang 1 3)
  );_end_while

  (mapcar 'setvar sv_lst sv_vals)
  (princ)
);_end_defun
(princ)

;(mapcar '(lambda (x) (setq txt (strcat txt "\n" x))) (list "qwerty" "asdfgh"))

;;----------------------------------------------------------------------;;
;;                             End of File                              ;;
;;----------------------------------------------------------------------;;